function [C,x] = find_channel_capacity(P, xc, lim)
% compute the channel capacity of a channel specified by matrix P via
% convex optimization
%
% [C,x] = channel_capacity(P)
% dim(P) = W x V, where W = number of output symbols and n = number of
% input symbols (s.t. y = P*x)
% C is the capacity
% x is the capacity achieving input distribution
% xc are the actual values of x

n = size(P,2); % input 
m = size(P,1); % output

% normalize
P = P./repmat(sum(P),m,1); 

% This is P tilde
Pt = P.*log2(P);
Pt(isnan(Pt)) = 0;
Pt = sum(Pt)';



% Channel capacity 
switch lim
    case 'none'
        cvx_begin
            variable x(n)     
    
            maximize (Pt'*x + sum(entr(P*x))/log(2))
            subject to
                x >= 0;
                sum(x) == 1; 
        cvx_end
        C = cvx_optval; 
    case 'avgPwr'
        cvx_begin
            variable x(n)     
    
            maximize (Pt'*x + sum(entr(P*x))/log(2))
            subject to
                x >= 0;
                sum(x) == 1;
                (xc.^2)'*x <= 1;
        cvx_end
        C = cvx_optval;
    case 'meanValue'
        cvx_begin
            variable x(n)     
    
            maximize (Pt'*x + sum(entr(P*x))/log(2))
            subject to
                x >= 0;
                sum(x) == 1;
                xc'*x <= 1;
        cvx_end
        C = cvx_optval;
    otherwise
        disp('Unknown method.')
end
        

% Results
display(['The channel capacity is: ' num2str(C) ' bits.'])

end
